from payu.profilers.gprof import Gprof
from payu.profilers.darshan import Darshan
from payu.profilers.oss import OpenSpeedShop

index = {
    'gprof': Gprof,
    'darshan': Darshan,
    'oss': OpenSpeedShop,
    'openspeedshop': OpenSpeedShop,
}
